#!/usr/bin/perl -w
use strict;
use Ace;

## Turn off output caching
$| = 9;

my $acedb = do "../etc/acedb.pl";
my $conf = do "../etc/wormi.pl";

print STDERR "Connecting AceDB...";
my $db = Ace->connect(%{$acedb}) || die "Connection failure", Ace->error;
print STDERR " Done!\n";

my $best_hit_file = $conf->{DIR_WORMI}.$conf->{DIR_WORMBASE_DATA}
                .sprintf($conf->{FILE_WS_BLAST_HITS}, $conf->{WORMBASE_VER});
my $wi_homolog_file = $conf->{DIR_WORMI}.$conf->{DIR_WORMBASE_DATA}
                .$conf->{FILE_WS_HOMOLOG}.".".$conf->{WORMBASE_VER}.".txt";

open(OUT, ">$wi_homolog_file");
open(BEST, $best_hit_file);
while(<BEST>) {
    chomp;
    my @tmp = split(/,/);

    my $prot_query = $db->fetch(Protein => "WP:$tmp[0]");
    if(! $prot_query ) {
        print STDERR $tmp[0]," is NOT AVAILABLE!! Report to WormBase!\n";
        next;
    }

    my @homol_data = $prot_query->Pep_homol(1);
    my $prot_query_length = length($prot_query->Peptide->right(1));

    for my $i (1,3,5,7,9,11,13) {
        if($tmp[$i]) {
            my $prot = $db->fetch(Protein => $tmp[$i]);
            if(! $prot ) {
                print STDERR $tmp[$i]
                        ," is NOT AVAILABLE!! Report to WormBase!\n";
                next;
            }

            my $source_matched = 0;
            foreach my $tmp_h (@homol_data) {
                if($tmp_h ne $tmp[$i]) { next; }
                $source_matched += $tmp_h->right(4) - $tmp_h->right(3);
                while( my $tmp_down = $tmp_h->right(3)->down ) {
                    $source_matched += $tmp_down->right(1) - $tmp_down;
                    $tmp_h = $tmp_down;
                }
            }

            ## C. elegans
            if($i == 1 ) {
                if(! $prot->Corresponding_CDS ) {
                    print STDERR $prot
                        ," is NOT AVAILABLE!! Report to WormBase!\n";
                    next;
                }
                    
                print OUT $tmp[0],"\t",$tmp[$i],"\t",$tmp[$i+1],"\t"
                    , $prot->Species, "\t"
                    , $prot->Corresponding_CDS->DB_remark,"\t"
                    , sprintf("%.3f", $source_matched/$prot_query_length), "\n";
            ## C. briggsae
            } elsif($i == 5 ) {
                print OUT $tmp[0],"\t",$tmp[$i],"\t",$tmp[$i+1],"\t"
                    , $prot->Species, "\t"
                    , $prot->Corresponding_CDS,"\t"
                    , sprintf("%.3f", $source_matched/$prot_query_length), "\n";
            ## Others
            } else {
                print OUT $tmp[0],"\t",$tmp[$i],"\t",$tmp[$i+1],"\t"
                    , $prot->Species, "\t"
                    , $prot->Description,"\t"
                    , sprintf("%.3f", $source_matched/$prot_query_length), "\n";
                
            }
        }
    }
}
close(BEST);
close(OUT);
