#!/usr/bin/perl -w
use strict;
use Ace;

## Turn off output caching
$| = 9;

my $conf = do "../etc/wormi.pl";
my $acedb = do "../etc/acedb.pl";

my %wormgene;
my $file_gene = $conf->{DIR_WORMI}.$conf->{DIR_WORMBASE_DATA}
                .sprintf($conf->{FILE_WS_GENEID}, $conf->{WORMBASE_VER});
open(GENE,$file_gene);
print STDERR "Read WormGene ID from ",$file_gene,"...\n";
while(<GENE>) {
    chomp;
    my ($geneid,@tmp) = split(/,/);
    $wormgene{$geneid} = 1;
}
close(GENE);

print STDERR "Connecting AceDB...";
my $db = Ace->connect(%{$acedb}) || die "Connection failure", Ace->error;
print STDERR " Done!\n";

my $file_output = $conf->{DIR_WORMI}.$conf->{DIR_WORMBASE_DATA}
                    .$conf->{FILE_WS_RNAI}.".".$conf->{WORMBASE_VER}.".txt";
open(RNAI,">$file_output");
print STDERR "Write ",$file_output,"...\n";

foreach my $tmp_gene (sort keys %wormgene) {
    my $wbgene = $db->fetch(Gene => $tmp_gene);
    my @rnai = $wbgene->get('RNAi_result');
    next unless (@rnai);
    foreach my $tmp_rnai (@rnai) {
        my $target = $tmp_rnai->right(2);
        my $wbrnai = $db->fetch(RNAi => $tmp_rnai);
        my @phenotype = $wbrnai->Phenotype();
        foreach my $tmp_ph (@phenotype) {
            my @last_names;
            foreach my $tmp_lab ($wbrnai->Laboratory) {
                push(@last_names, $tmp_lab->Representative->Last_name);
            }
           
            if( $wbrnai->PCR_product() ) {
                foreach my $tmp_pcr ($wbrnai->PCR_product()) {
                    if( $wbrnai->Sequence() ) {
                        foreach my $tmp_seq ($wbrnai->Sequence()) {
                            print RNAI $tmp_rnai,"\t", $tmp_pcr,"\t",
                                $tmp_seq,"\t", $tmp_ph,"\t",
                                join(";",@last_names),"\t", 
                                $wbrnai->Reference,"\t", 
                                $wbgene,"\t", $target,"\n";
                        }
                    } else {
                        my $tmp_seq = "";
                        print RNAI $tmp_rnai,"\t", $tmp_pcr,"\t",
                                $tmp_seq,"\t", $tmp_ph,"\t",
                                join(";",@last_names),"\t", 
                                $wbrnai->Reference,"\t", 
                                $wbgene,"\t", $target,"\n";
                    }
                }
            } else {
                my $tmp_pcr = "";
                foreach my $tmp_seq ($wbrnai->Sequence()) {
                        print RNAI $tmp_rnai,"\t", $tmp_pcr,"\t",
                            $tmp_seq,"\t", $tmp_ph,"\t",
                            join(";",@last_names),"\t", 
                            $wbrnai->Reference,"\t", 
                            $wbgene,"\t", $target,"\n";
                }
            }

        }
    }
}
close(RNAI);

