#!/usr/bin/perl -w
use strict;

my $conf = do "../etc/wormi.pl";
my @tmp_ts = localtime(time);
my $timestamp = sprintf("%02d%02d%02d",$tmp_ts[3],$tmp_ts[4]+1,$tmp_ts[5]%100);

my $DIR_WS = $conf->{DIR_WORMI}. $conf->{DIR_WORMBASE_DATA};
my $DIR_WI = $conf->{DIR_WORMI}. $conf->{DIR_WORMI_DATA};
my $SUFFIX_WI = ".".$timestamp.".". $conf->{WORMBASE_VER}.".txt";

my $WS_RNAI_FILE = $DIR_WS . $conf->{FILE_WS_RNAI}.".". $conf->{WORMBASE_VER}
                    .".txt";
my $WS_PAPER_FILE = $DIR_WS . $conf->{FILE_WS_PAPER}.".". $conf->{WORMBASE_VER}
                    .".txt";
my $WS_GFF_GENE = $DIR_WS. sprintf($conf->{FILE_WS_GFF}, 
                    $conf->{WORMBASE_VER}.".gene");
my $WS_GFF_RNAI = $DIR_WS. sprintf($conf->{FILE_WS_GFF},
                    $conf->{WORMBASE_VER}.".RNAi_reagent");
                    
my $WI_REAGENT_FILE     = $DIR_WI. $conf->{FILE_WORMI_REAGENT}.$SUFFIX_WI;
my $WI_REAGENT_GENE_FILE = $DIR_WI. $conf->{FILE_WORMI_REAGENT_GENE}.$SUFFIX_WI;
my $WI_LIBRARY_FILE     = $DIR_WI. $conf->{FILE_WORMI_LIBRARY}.$SUFFIX_WI;
my $WI_LIB_REAGENT_FILE = $DIR_WI. $conf->{FILE_WORMI_LIB_REAGENT}.$SUFFIX_WI;
my $WI_EXPERIMENT_FILE  = $DIR_WI. $conf->{FILE_WORMI_EXPERIMENT}.$SUFFIX_WI;
my $WI_EXP_REAGENT_FILE = $DIR_WI. $conf->{FILE_WORMI_EXP_REAGENT}.$SUFFIX_WI;
my $WI_EXP_PHE_FILE     = $DIR_WI. $conf->{FILE_WORMI_EXP_PHENOTYPE}.$SUFFIX_WI;
my $WI_SCREEN_FILE      = $DIR_WI. $conf->{FILE_WORMI_SCREEN}.$SUFFIX_WI;

my (%reagent, %lib_reagent, %experiment, %screen);

my %paper;
open(PAPER, $WS_PAPER_FILE);
print STDERR "Read ",$WS_PAPER_FILE,"...\n";
while(<PAPER>) {
    chomp;
    my ($wbpaper, $author, $year, $citation) = split(/\t/);
    next unless($author);
    $paper{$wbpaper}->{year} = $year;
    $paper{$wbpaper}->{author} = $author;

    my $first_author = $author;
    if( $citation =~ /([0-9A-Za-z\-\*\.\, ]+)\(/ ) {
        $first_author = $1;
        $first_author =~ s/et al\.//g;
        $first_author =~ s/-COMMENT//g;
        $first_author =~ s/\-\*\.\,[0-9]//g;
        $first_author =~ s/\s+$//;
    }
    my $screenName = $first_author.$year.":".$wbpaper;
    $screenName =~ s/\s+//g;
    $screenName =~ s/WBPaper[0]+//g;
    $screenName =~ s/et\.al//g;
    $paper{$wbpaper}->{screenName} = $screenName;
    $screen{$screenName}->{author} = $first_author;
    $screen{$screenName}->{description} = $citation;
    $screen{$screenName}->{beginDate} = $year."-00-00";
    $screen{$screenName}->{endDate} = $year."-00-00";
}
close(PAPER);

my (%gene, %rnai_reagent);
my $gene_idx = 1;
open(GENE, $WS_GFF_GENE);
print STDERR "Read ",$WS_GFF_GENE,"...\n";
while(<GENE>) {
    my ($seqname, $source, $feature, $start, $end, $score,
            $strand, $frame, $comment) = split(/\t/);
    if( $comment =~ /(WBGene[0-9]+)/ ) {
        my $gene_id = $1;
        $gene{$gene_id}->{chr} = $seqname;
        $gene{$gene_id}->{start} = $start;
        $gene{$gene_id}->{end} = $end;
        $gene{$gene_id}->{length} = $end - $start + 1;
        $gene{$gene_id}->{idx} = $gene_idx;
        $gene_idx++;
    }
}
close(GENE);

open(RNAIR, $WS_GFF_RNAI);
print STDERR "Read ",$WS_GFF_RNAI,"...\n";
while(<RNAIR>) {
    my ($seqname, $source, $feature, $start, $end, $score,
            $strand, $frame, $comment) = split(/\t/);
    if( $comment =~ /(WBRNAi[0-9]+)/ ) {
        my $rnai_id = $1;
        $source =~ s/RNAi_//;
        push(@{$rnai_reagent{$rnai_id}->{$source}},
                { start => $start, end => $end, chr => $seqname });
    }
}
close(RNAIR);

my %exists_screen;
open(RNAI,$WS_RNAI_FILE);
print STDERR "Read ",$WS_RNAI_FILE,"...\n";
while(<RNAI>) {
    chomp;
    my ($wbrnai, $pcr_product, $sequence, $phenotype, $screen_author,   
            $wbpaper, $wbgene, $target_type) = split(/\t/);

    next unless ($target_type);
    $target_type =~ s/RNAi_//;

    my $target_gene = $gene{$wbgene};
    my $gene_idx = 0;
    my $match_length = 0;
    if( $target_gene->{length} && $target_gene->{length} > 0 ) {
        foreach my $tmp_r (@{$rnai_reagent{$wbrnai}->{$target_type}}) {
            next if( $target_gene->{chr} ne $tmp_r->{chr} );
            next if( $tmp_r->{end} < $target_gene->{start} );
            next if( $tmp_r->{start} > $target_gene->{end} );

            if( $target_gene->{end} < $tmp_r->{end} ) {
                $match_length += $target_gene->{end} + 1;
            } else {
                $match_length += $tmp_r->{end} + 1;
            }

            if( $target_gene->{start} > $tmp_r->{start} ) {
                $match_length -= $target_gene->{start};
            } else {
                $match_length -= $tmp_r->{start};
            }
        }
        
        $match_length = sprintf("%.3f", $match_length/$target_gene->{length});
        $gene_idx = $target_gene->{idx};
        
    } else {
        print STDERR "$wbgene is not available on GFF file. Report Wormbase!\n";
    }

    if( $pcr_product ) {
        $reagent{$pcr_product}->{reagentType} = "pcr_product";
        $reagent{$pcr_product}->{reagentTemplate} = "unknown";
        if( $pcr_product =~ /sjj_/ || $pcr_product =~ /cenix/ ) {
            $reagent{$pcr_product}->{reagentTemplate} = "gDNA";
        } elsif( $pcr_product =~ /mv_/ || $pcr_product =~ /^yk/) {
            $reagent{$pcr_product}->{reagentTemplate} = "cDNA";
        }
        $reagent{$pcr_product}->{target}->{$wbgene} 
            =   { gene_id => $wbgene, target_type => $target_type,
                  target_position => $gene_idx, match_length => $match_length };
        
        $lib_reagent{$wbrnai}->{reagentName} = $pcr_product;
        push(@{$experiment{$wbrnai}->{reagent}}, $pcr_product);
    } elsif ( $sequence ) {
        $reagent{$sequence}->{reagentType} = "sequence";
        $reagent{$sequence}->{reagentTemplate} = "unknown";
        if( $sequence =~ /^yk/ ) {
            $reagent{$sequence}->{reagentTemplate} = "cDNA";
        }

        $reagent{$sequence}->{target}->{$wbgene} 
            =   { gene_id => $wbgene, target_type => $target_type,
                  target_position => $gene_idx, match_length => $match_length };
        $lib_reagent{$wbrnai}->{reagentName} = $sequence;
        push(@{$experiment{$wbrnai}->{reagent}}, $sequence);
    }
    
    push(@{$experiment{$wbrnai}->{phenotypeClass}}, $phenotype);
    $experiment{$wbrnai}->{experimentBy} = $screen_author;

    if( $wbpaper ) {
        my $screen_name = $paper{$wbpaper}->{screenName};
        if( $screen_name ) {
            $exists_screen{$screen_name} = 1;
            $experiment{$wbrnai}->{screenName} = $screen_name;
            if( $screen_author ) {
                $screen{$screen_name}->{author} = $screen_author;
            }
        } else {
            $experiment{$wbrnai}->{screenName} = $wbpaper;
        }
    } else {
        $experiment{$wbrnai}->{screenName} = "unknown";
    }
   
    if( $screen_author ) {
        $experiment{$wbrnai}->{experimentBy} = $screen_author;
    } else {
        $experiment{$wbrnai}->{experimentBy} = "unknown";
    }
}
close(RNAI);

open(REAGENTGENE, ">$WI_REAGENT_GENE_FILE");
print STDERR "Write ",$WI_REAGENT_GENE_FILE,"...\n";
foreach my $reagentName (sort keys %reagent) {
    my $r = $reagent{$reagentName};
    foreach my $tmp_wbgene (sort keys %{$r->{target}}) {
        my $tmp_t = $r->{target}->{$tmp_wbgene};
        print REAGENTGENE $reagentName,"\t", $reagentName,"\t", $tmp_wbgene
                ,"\t", $tmp_t->{target_type},"\t", $tmp_t->{target_position}
                ,"\t",$tmp_t->{match_length},"\t", $conf->{WORMBASE_VER},"\n";
    }
}
close(REAGENTGENE);

open(REAGENT, ">$WI_REAGENT_FILE");
print STDERR "Write ",$WI_REAGENT_FILE,"...\n";
foreach my $reagentName (sort keys %reagent) {
    my $r = $reagent{$reagentName};
    print REAGENT $reagentName,"\t", $r->{reagentType}
            ,"\t", $r->{reagentTemplate},"\t", $conf->{WORMBASE_VER},"\n";
}
close(REAGENT);

open(LIBRARY, ">$WI_LIBRARY_FILE");
print STDERR "Write ",$WI_LIBRARY_FILE,"...\n";
print LIBRARY $conf->{WORMBASE_VER},"\t",$conf->{WORMBASE_VER},"\n";
close(LIBRARY);

my %exists_lib_reagent;
open(LIBREAGENT, ">$WI_LIB_REAGENT_FILE");
print STDERR "Wirte ",$WI_LIB_REAGENT_FILE,"...\n";
foreach my $tmp_wbrnai (sort keys %lib_reagent) {
    my $tmp_reagentName = $lib_reagent{$tmp_wbrnai}->{reagentName};
    if(exists $exists_lib_reagent{$tmp_reagentName}) { next; }
    else { $exists_lib_reagent{$tmp_reagentName} = 1; }

    print LIBREAGENT $lib_reagent{$tmp_wbrnai}->{reagentName}
            ,"\t", $conf->{WORMBASE_VER},
            ## Well, Plate, Sequencing, clone_status will be updated
            ,"\t","0","\t","X00","\t","unknown","\t","unknown","\n";
}
close(LIBREAGENT);

open(SCREEN, ">$WI_SCREEN_FILE");
print STDERR "Write ",$WI_SCREEN_FILE,"...\n";
foreach my $tmp_sname (sort keys %screen) {
    my $tmp_s = $screen{$tmp_sname};
    next if(not exists $exists_screen{$tmp_sname} );
    print SCREEN $tmp_sname,"\t", $tmp_s->{description}
            ,"\t", $tmp_s->{author},"\t", "unknown"
            ,"\t", "unknown", "\t", "unknown"
            ,"\t", $tmp_s->{beginDate}, "\t", $tmp_s->{endDate},"\n";
}
close(SCREEN);

my %exp_reagent;
open(EXPREAGENT, ">$WI_EXP_REAGENT_FILE");
print STDERR "Write ",$WI_EXPERIMENT_FILE,"...\n";
foreach my $tmp_wbrnai (sort keys %experiment) {
    my $tmp_e = $experiment{$tmp_wbrnai};
    foreach my $tmp_r (@{$tmp_e->{reagent}}) {
        if(exists $exp_reagent{$tmp_wbrnai} 
            && $exp_reagent{$tmp_wbrnai} eq $tmp_r) { 
            next;
        }
        my $exp_info = $conf->{WORMBASE_VER}.":".$tmp_r;
        $exp_reagent{$tmp_wbrnai}->{info} = $exp_info;
        print EXPREAGENT $tmp_r,"\t", $conf->{WORMBASE_VER}
                ,"\t", $tmp_wbrnai,"\t",$exp_info,"\n";
    }
}
close(EXPREAGENT);

my %check_exp_phe;
open(EXP, ">$WI_EXPERIMENT_FILE");
open(EXPPHE, ">$WI_EXP_PHE_FILE");
print STDERR "Write ",$WI_EXPERIMENT_FILE,"...\n";
print STDERR "Write ",$WI_EXP_PHE_FILE,"...\n";
foreach my $tmp_wbrnai (sort keys %experiment) {
    my $tmp_e = $experiment{$tmp_wbrnai};
    print EXP $tmp_wbrnai,"\t", $tmp_e->{screenName}
            ,"\t", $exp_reagent{$tmp_wbrnai}->{info},"\t"
            ,$tmp_e->{experimentBy},"\n";
    foreach my $tmp_phe (@{$tmp_e->{phenotypeClass}}) {
        if( exists $check_exp_phe{$tmp_wbrnai}->{$tmp_phe} ) {
            next; 
        }
        print EXPPHE $tmp_wbrnai,"\t", $tmp_phe,"\t"
                ,"\t", $exp_reagent{$tmp_wbrnai}->{info}
                ,"\t","unknown\n";
        $check_exp_phe{$tmp_wbrnai}->{$tmp_phe} = 1;
    }
}
close(EXP);
close(EXPPHE);
