#!/usr/bin/perl -w
use strict;

my $conf_sp     = do "../etc/celegans.pl";

my $FILE_TMPL     = '../data/'.$conf_sp->{SPECIES}.'.%s.'.$conf_sp->{SOURCE};
my $FILE_WIN_TMPL = '../data/'.$conf_sp->{SPECIES}.'.%d.%s.'.$conf_sp->{SOURCE};

my (%repeat_win, %gene_win);
my (%gene_map, %pcr_product_map, %reagent_map);

foreach my $gff_file (`ls ../gff/$conf_sp->{SPECIES}/*.gff`) {
    chomp($gff_file);
    open(GFF,$gff_file);
    print STDERR "Read ",$gff_file,"...\n";
    while(<GFF>) {
        chomp;
        my ($chr, $source, $feature, $start, $end, $score,
            $strand, $frame, $comment) = split(/\t/);
        
        if( $feature eq "inverted_repeat" 
                || $feature eq "tandem_repeat"
                || $feature eq "repeat_region" ) {
            foreach my $tmp_win (@{$conf_sp->{WINDOW_SIZES}}) {
                my $idx = int($start/$tmp_win/1000);
                if(exists $repeat_win{$tmp_win}->{$chr}->[$idx]->{$feature}) {
                    $repeat_win{$tmp_win}->{$chr}->[$idx]->{$feature}++;
                } else {
                    $repeat_win{$tmp_win}->{$chr}->[$idx]->{$feature} = 1;
                }
                if(exists $repeat_win{$tmp_win}->{$chr}->[$idx]->{'repeat_all'}) {
                    $repeat_win{$tmp_win}->{$chr}->[$idx]->{'repeat_all'}++;
                } else {
                    $repeat_win{$tmp_win}->{$chr}->[$idx]->{'repeat_all'} = 1;
                }
            }
        } elsif( $feature eq "gene" ) {
            if( $comment =~ /WBGene/ ) {
                foreach my $tmp_win (@{$conf_sp->{WINDOW_SIZES}}) {
                    my $idx = int($start/$tmp_win/1000);
                    if(exists $gene_win{$tmp_win}->{$chr}->[$idx] ) {
                        $gene_win{$tmp_win}->{$chr}->[$idx]->{'gene'}++;
                    } else {
                        $gene_win{$tmp_win}->{$chr}->[$idx]->{'gene'} = 1;
                    }
                }
            }
        } elsif( $feature eq "Transcript" ) {

            if( $comment =~ /Transcript \"([A-Za-z0-9\.\:]+)\"/ ) {
                $gene_map{$1}->{$chr}->{$start} = 1;
            }
            if( $comment =~ /WormPep \"([A-Za-z0-9\.\:]+)\"/ ) {
                $gene_map{$1}->{$chr}->{$start} = 1;
            }
            if( $comment =~ /Gene \"([A-Za-z0-9\.\:]+)\"/ ) {
                $gene_map{$1}->{$chr}->{$start} = 1;
            }
            if( $comment =~ /CDS \"([A-Za-z0-9\.\:]+)\"/ ) {
                $gene_map{$1}->{$chr}->{$start} = 1;
            }
        } elsif( $feature eq "PCR_product" ) {
            if( $comment =~ /PCR_product \"([A-Za-z0-9\.\:\-\_]+)\"/ ) {
                my $name = $1;
                if( $name =~ /(sjj_[A-Za-z0-9\.]+)_[0-9]+/ ) { 
                    $name = $1;
                }
                $pcr_product_map{$name}->{$chr}->{$start} = 1;
            }
        } elsif( $feature eq "reagent" ) {
            if( $comment =~ /Oligo_set \"([A-Za-z0-9\.\:\-\_]+)\"/ ) {
                my $name = $1;
                $reagent_map{$name}->{$chr}->{$start} = 1;
            }
        }
    }
    close(GFF);
}

&write_win_file(\%repeat_win,'inverted_repeat');
&write_win_file(\%repeat_win,'tandem_repeat');
&write_win_file(\%repeat_win,'repeat_region');
&write_win_file(\%repeat_win,'repeat_all');
&write_win_file(\%gene_win,'gene');

&write_map_file(\%gene_map,'gene');
&write_map_file(\%pcr_product_map, 'pcr_product');
&write_map_file(\%reagent_map, 'oligo_set');

sub write_win_file {
    my ($data,$tag) = @_;
    foreach my $tmp_win (@{$conf_sp->{WINDOW_SIZES}}) {
        my $file_out = sprintf($FILE_WIN_TMPL, $tmp_win, $tag);
        open(OUT, ">$file_out");
        print STDERR "Write ",$file_out,"...\n";
        foreach my $tmp_chr (@{$conf_sp->{CHR}}) {
            my $idx = 0;
            foreach my $tmp (@{$data->{$tmp_win}->{$tmp_chr}}) {
                my $value = 0;
                if($tmp->{$tag}) { $value = $tmp->{$tag}; }

                my $pos = $idx * $tmp_win * 1000;
                print OUT $tmp_chr,"\t",$pos,"\t",$value,"\n";
                $idx++;
            }
        }
        close(OUT);
    }
}

sub write_map_file {
    my ($data,$tag) = @_;
    
    my $file_out = sprintf($FILE_TMPL, $tag);
    open(OUT, ">$file_out");
    print STDERR "Write ",$file_out,"...\n";
    
    foreach my $tmp_name (sort keys %{$data}) {
        foreach my $tmp_chr (sort keys %{$data->{$tmp_name}}) {
            foreach my $tmp_start 
                            (sort keys %{$data->{$tmp_name}->{$tmp_chr}}) {
                print OUT $tmp_name,"\t",$tmp_chr,"\t",$tmp_start,"\n"; 
            }
        }
    }
    close(OUT);
}
