#!/usr/bin/perl -w
use strict;

use lib "../libperl";
use GenomeMap::SVG;
use GenomeMap::IO;

my $svg = new GenomeMap::SVG;
my $io  = new GenomeMap::IO;

my $WINDOW_SIZE = 500;              ## 100 kbp, 250 kbp, or 500 kbp
my $REPEAT_TYPE = 'repeat_all';     ## All repeats
my $GENE_TYPE   = 'transcript';
my $SPECIES     = 'celegans';       ## C. elegans
#my $TITLE       = $SPECIES.' : M value';

my $conf = do "../etc/expression.pl";
my $conf_species = do "../etc/$SPECIES.pl";

my $file_gene = sprintf("../data/%s.%d.%s.%s", $SPECIES,$WINDOW_SIZE,
                        $GENE_TYPE,$conf_species->{SOURCE});
my $file_repeat = sprintf("../data/%s.%d.%s.%s", $SPECIES,$WINDOW_SIZE,
                        $REPEAT_TYPE,$conf_species->{SOURCE});

my $gene_clust = $io->read_chr_stat($file_gene);
my $rep_clust = $io->read_chr_stat($file_repeat);

## Estimate parameters from data
my $chr_count = 0;
my $figure_width = 0;
my $max_repeat_count = 0;
my $max_gene_count = 0;
foreach my $tmp_chr (@{$conf_species->{CHR}}) {
    my $tmp_width = $conf_species->{CHR_LENGTH}->{$tmp_chr}
                        / $conf->{CHR_BOX_X_SCALE};
    if( $figure_width < $tmp_width ) { $figure_width = $tmp_width; }
    
    if( $rep_clust->{$tmp_chr}->{'max'} > $max_repeat_count ) {
        $max_repeat_count = $rep_clust->{$tmp_chr}->{'max'};
    }

    if( $gene_clust->{$tmp_chr}->{'max'} > $max_gene_count ) {
        $max_gene_count = $gene_clust->{$tmp_chr}->{'max'};
    }

    $chr_count++;
}
$figure_width += $conf->{CHR_BOX}->{x};
my $figure_height = $chr_count * $conf->{CHR_STEP_Y} + $conf->{TOP_MARGIN};

## HEADER
print $svg->http_header();
print $svg->header( -width => $figure_width, -height => $figure_height ),"\n";

## TITLE
#print $svg->text( -x => $conf->{TITLE}->{'x'}, -y => $conf->{TITLE}->{'y'},
#                  -text => $TITLE, -size => $conf->{TITLE}->{'font_size'},
#                  -font => $conf->{FONT}),"\n";

## CLUSTER INDICATOR
my $height_indicator = $conf->{CLUST_INDICATE}->{box_height};
print $svg->rect(-x => $conf->{CLUST_INDICATE}->{box_x},
                 -y => $conf->{CLUST_INDICATE}->{y_gene}-$height_indicator,
                 -width => $conf->{CLUST_INDICATE}->{box_width},
                 -height => $height_indicator,
                 -style  => $conf->{GENE_CLUST}->{style}),"\n";
print $svg->text(-x => $conf->{CLUST_INDICATE}->{x}, 
                 -y => $conf->{CLUST_INDICATE}->{y_gene}, 
                 -text => sprintf("%s (window=%d kb)",$GENE_TYPE,$WINDOW_SIZE),
                 -font => $conf->{FONT},
                 -size => $conf->{CLUST_INDICATE}->{font_size}),"\n";

print $svg->rect(-x => $conf->{CLUST_INDICATE}->{box_x},
                 -y => $conf->{CLUST_INDICATE}->{y_repeat}-$height_indicator,
                 -width => $conf->{CLUST_INDICATE}->{box_width},
                 -height => $height_indicator,
                 -style  => $conf->{REPEAT_CLUST}->{style}),"\n";
print $svg->text(-x => $conf->{CLUST_INDICATE}->{x}, 
                 -y => $conf->{CLUST_INDICATE}->{y_repeat}, 
                 -text =>sprintf("%s (window=%d kb)",$REPEAT_TYPE,$WINDOW_SIZE),
                 -font => $conf->{FONT},
                 -size => $conf->{CLUST_INDICATE}->{font_size}),"\n";

my $chr_idx = 0;
foreach my $tmp_chr (@{$conf_species->{CHR}}) {
    my $x_label = $conf->{CHR_LABEL}->{x};
    my $y_label = $conf->{CHR_LABEL}->{y_init} + $chr_idx*$conf->{CHR_STEP_Y};
    
    ## CHROMOSOME LABEL
    print "\n",$svg->comment("Chromosome Label : ".$tmp_chr),"\n";
    print $svg->text(   -id => 'label_chr'.$tmp_chr, 
                        -x => $x_label, -y => $y_label,
                        -font => $conf->{FONT}, 
                        -size => $conf->{CHR_LABEL}->{font_size},
                        -text => 'Chr '.$tmp_chr),"\n";

    ## CHROMOSOME BOX
    print "\n",$svg->comment("Chromosome Box : ".$tmp_chr),"\n";
    my $x_chr_box = $conf->{CHR_BOX}->{x};
    my $y_chr_box = $conf->{CHR_BOX}->{y_init} + $chr_idx*$conf->{CHR_STEP_Y};
    my $width_chr_box = $conf_species->{CHR_LENGTH}->{$tmp_chr}
                        / $conf->{CHR_BOX_X_SCALE};
    print $svg->rect( -id => 'chr_box_'.$tmp_chr,
                    -x => $x_chr_box, -y => $y_chr_box, 
                    -width => $width_chr_box,
                    -height => $conf->{CHR_BOX}->{height},
                    -style => $conf->{CHR_BOX}->{'style'}),"\n";

    ## Y GRID on CHROMOSOME BOX
    print "\n",$svg->comment("Y Grid on Chromosome Box : ".$tmp_chr),"\n";
    my $y_chr_box_grid = $y_chr_box + $conf->{CHR_BOX_Y}->{y_step};
    my $y_idx = 0;
    while($y_chr_box_grid < $y_chr_box + $conf->{CHR_BOX}->{height}) {
        print $svg->hline(-x => $x_chr_box, -y => $y_chr_box_grid, 
                        -length => $width_chr_box, 
                        -style => $conf->{CHR_BOX_Y}->{'line_style'}),"\n";
        my $x_grid_label = $x_chr_box + $conf->{CHR_BOX_Y}->{x_margin};
        my $y_grid_label = $y_chr_box_grid + $conf->{CHR_BOX_Y}->{y_margin}
                            + ($conf->{CHR_BOX_Y}->{y_step}/2);
        print $svg->text(-x => $x_grid_label, -y => $y_grid_label,
                        -font => $conf->{FONT},
                        -size=> $conf->{CHR_BOX_Y}->{'label_font_size'},
                        -text => $conf->{CHR_BOX_Y}->{label}->[$y_idx]),"\n";
        $y_chr_box_grid += $conf->{CHR_BOX_Y}->{y_step};
        $y_idx++;
    }

    ## X GRID ON CHROMOSOME BOX
    print "\n",$svg->comment("X Grid on Chromosome Box : ".$tmp_chr),"\n";
    my $x_grid_step = $conf->{CHR_BOX_X_STEP}/ $conf->{CHR_BOX_X_SCALE}; 
    my $x_chr_box_grid = $x_chr_box + $x_grid_step;
    while($x_chr_box_grid < $x_chr_box + $width_chr_box) {
        print $svg->vline(-x => $x_chr_box_grid, -y => $y_chr_box, 
                        -length => $conf->{CHR_BOX}->{height}, 
                        -style => $conf->{CHR_BOX_X}->{'line_style'}),"\n";
        my $x_grid_label = sprintf("%d M",($x_chr_box_grid - $x_chr_box) 
                            * $conf->{CHR_BOX_X_SCALE} / 1000000);
        print $svg->text(-x => $x_chr_box_grid+$conf->{CHR_BOX_X}->{x_margin}, 
                        -y => $y_chr_box + $conf->{CHR_BOX_X}->{y_margin},
                        -font => $conf->{FONT},
                        -size=> $conf->{CHR_BOX_X}->{'label_font_size'},
                        -text => $x_grid_label),"\n";
        $x_chr_box_grid += $x_grid_step;
    }

    ## Gene Cluster
    print "\n",$svg->comment("Gene Cluster : ".$tmp_chr),"\n";
    my $clust_width = $WINDOW_SIZE * 1000 / $conf->{CHR_BOX_X_SCALE};
    my $clust_bg_y = $conf->{GENE_CLUST}->{y_init}+$chr_idx*$conf->{CHR_STEP_Y};
    print $svg->rect(-x => $x_chr_box, -y => $clust_bg_y,
                    -width => $width_chr_box,
                    -height => $conf->{GENE_CLUST}->{max_height},
                    -style => $conf->{GENE_CLUST}->{'background_style'}),"\n";

    foreach my $tmp_pos (sort { $a <=> $b } 
                            keys %{$gene_clust->{$tmp_chr}->{data}}) {
        my $clust_x = $conf->{GENE_CLUST}->{'x'}
                        + $tmp_pos / $conf->{CHR_BOX_X_SCALE};
        my $tmp_width = $clust_width;
        if( $clust_x + $tmp_width > $width_chr_box+$conf->{CHR_BOX}->{x}) {
            $tmp_width = $width_chr_box + $conf->{CHR_BOX}->{x} - $clust_x ;
        }
        my $clust_height = $gene_clust->{$tmp_chr}->{data}->{$tmp_pos}
                        / $max_gene_count
                        * $conf->{GENE_CLUST}->{max_height};
        my $clust_y = $conf->{GENE_CLUST}->{max_height}
                        + $conf->{GENE_CLUST}->{y_init} 
                        + $conf->{CHR_STEP_Y}*$chr_idx - $clust_height;
        print $svg->rect(-x => $clust_x, -y => $clust_y, 
                        -width => $tmp_width , -height => $clust_height, 
                        -style => $conf->{GENE_CLUST}->{'style'}),"\n";
    }

    ## Repeat Cluster
    print "\n",$svg->comment("Repeat Cluster : ".$tmp_chr),"\n";
    
    ### Background
    $clust_bg_y = $conf->{REPEAT_CLUST}->{y_init}+$chr_idx*$conf->{CHR_STEP_Y};
    print $svg->rect(-x => $x_chr_box, -y => $clust_bg_y,
                    -width => $width_chr_box,
                    -height => $conf->{REPEAT_CLUST}->{max_height},
                    -style => $conf->{REPEAT_CLUST}->{'background_style'}),"\n";

    foreach my $tmp_pos (sort { $a <=> $b } 
                            keys %{$rep_clust->{$tmp_chr}->{data}}) {
        my $clust_x = $conf->{REPEAT_CLUST}->{'x'}
                        + $tmp_pos / $conf->{CHR_BOX_X_SCALE};
        my $tmp_width = $clust_width;
        if( $clust_x + $tmp_width > $width_chr_box+$conf->{CHR_BOX}->{x}) {
            $tmp_width = $width_chr_box + $conf->{CHR_BOX}->{x} - $clust_x ;
        }
        my $clust_height = 0;
        if( $rep_clust->{$tmp_chr}->{'max'} > 0 ) {
            $clust_height = $rep_clust->{$tmp_chr}->{data}->{$tmp_pos}
                        / $max_repeat_count
                        * $conf->{REPEAT_CLUST}->{max_height};
        }
        my $clust_y = $conf->{REPEAT_CLUST}->{max_height} 
                        + $conf->{REPEAT_CLUST}->{y_init} 
                        + $conf->{CHR_STEP_Y}*$chr_idx - $clust_height;
        print $svg->rect(-x => $clust_x, -y => $clust_y, 
                        -width => $tmp_width, -height => $clust_height, 
                        -style => $conf->{REPEAT_CLUST}->{'style'}),"\n";
    }

    print "\n";
    $chr_idx++;
}

## Print FOOTER
print $svg->footer(),"\n";
