#!/usr/bin/perl -w
use strict;

use lib "../libperl";
use GenomeMap::SVG;
use GenomeMap::IO;
use GenomeMap::Map;
use CGI;

my $svg = new GenomeMap::SVG;
my $io  = new GenomeMap::IO;
my $map = new GenomeMap::Map;
my $q   = new CGI;

my $GENE_TYPE   = 'gene';
my $SPECIES     = 'Celegans';       ## C. elegans

my $REPEAT_TYPE = 'repeat_all';
if($q->param('rep_type')
    && ($q->param('rep_type') eq 'repeat_all' 
        || $q->param('rep_type') eq 'inverted_repeat'
        || $q->param('rep_type') eq 'tandem_repeat'
        || $q->param('rep_type') eq 'repeat_region') ) {
    $REPEAT_TYPE = $q->param('rep_type');
}
        
my $WINDOW_SIZE = 100;              ## 100 kbp, 250 kbp, or 500 kbp
if($q->param('win_size') 
    && ($q->param('win_size') eq '100' || $q->param('win_size') eq '250'
        || $q->param('win_size') eq '500')) {
    $WINDOW_SIZE = $q->param('win_size');
}

#my $TITLE       = $SPECIES.' : M value';

my $conf = do "../etc/position.pl";
my $conf_species = do "../etc/$SPECIES.pl";

my $file_gene = sprintf("../data/%s.%d.%s.%s", $SPECIES,$WINDOW_SIZE,
                        $GENE_TYPE,$conf_species->{SOURCE});
my $file_repeat = sprintf("../data/%s.%d.%s.%s", $SPECIES,$WINDOW_SIZE,
                        $REPEAT_TYPE,$conf_species->{SOURCE});

my $gene_clust = $io->read_chr_stat($file_gene);
my $rep_clust = $io->read_chr_stat($file_repeat);

#my $file_data   = '../tmp/topTable_C1.txt';
my $file_data   = '../tmp/'.$q->param('file');
my $file_map    = "../data/$SPECIES.pcr_product.WS150";

my $p_value     = 1.00;
if( $q->param('P') ) { $p_value = $q->param('P'); }

my $color_value = 'yellow';
if( $q->param('color') ) { $color_value = $q->param('color'); }

my $file_type = 'eset';
if( $q->param('type') ) { $file_type = $q->param('type'); }

unless(-e $file_data) {
    die "NO DATA FILE : ",$file_data,"\n";
}

my $data_value  = undef;
if( $file_type eq 'topTable' ) {
        $data_value  = $io->read_topTable(-file => $file_data,
                                          -value => 'M', -P => $p_value );
} elsif ( $file_type eq 'eset' ) {
        $data_value  = $io->read_eset(-file => $file_data),
} else {
        $data_value  = $io->read_list(-file => $file_data),
}

my $with_curve = 0;
if($q->param('curve') && $q->param('curve') eq '1') { $with_curve = 1; }

my $data_map    = $io->read_map(-file => $file_map);
my $rv = $map->order_by_chr( -data => $data_value, -map => $data_map);

## Estimate parameters from data
my $chr_count = 0;
my $figure_width = 0;
my $max_repeat_count = 0;
my $max_gene_count = 0;
foreach my $tmp_chr (@{$conf_species->{CHR}}) {
    my $tmp_width = $conf_species->{CHR_LENGTH}->{$tmp_chr}
                        / $conf->{CHR_BOX_X_SCALE};
    if( $figure_width < $tmp_width ) { $figure_width = $tmp_width; }
    if( $conf_species->{CHR_LENGTH}->{$tmp_chr} < $conf->{CHR_BOX_X_SCALE} ) {
        next;
    }
    
    if( $rep_clust->{$tmp_chr}->{'max'} > $max_repeat_count ) {
        $max_repeat_count = $rep_clust->{$tmp_chr}->{'max'};
    }

    if( $gene_clust->{$tmp_chr}->{'max'} > $max_gene_count ) {
        $max_gene_count = $gene_clust->{$tmp_chr}->{'max'};
    }

    $chr_count++;
}
$figure_width += $conf->{CHR_BOX}->{x};
my $figure_height = $chr_count * $conf->{CHR_STEP_Y} + $conf->{TOP_MARGIN};

## HEADER
print $svg->http_header();
print $svg->header( -width => $figure_width, -height => $figure_height ),"\n";

## TITLE
#print $svg->text( -x => $conf->{TITLE}->{'x'}, -y => $conf->{TITLE}->{'y'},
#                  -text => $TITLE, -size => $conf->{TITLE}->{'font_size'},
#                  -font => $conf->{FONT}),"\n";

## CLUSTER INDICATOR
my $height_indicator = $conf->{CLUST_INDICATE}->{box_height};
print $svg->rect(-x => $conf->{CLUST_INDICATE}->{box_x},
                 -y => $conf->{CLUST_INDICATE}->{y_gene}-$height_indicator,
                 -width => $conf->{CLUST_INDICATE}->{box_width},
                 -height => $height_indicator,
                 -style  => $conf->{GENE_CLUST}->{style}),"\n";
print $svg->text(-x => $conf->{CLUST_INDICATE}->{x}, 
                 -y => $conf->{CLUST_INDICATE}->{y_gene}, 
                 -text => sprintf("%s (window=%d kb)",$GENE_TYPE,$WINDOW_SIZE),
                 -font => $conf->{FONT},
                 -size => $conf->{CLUST_INDICATE}->{font_size}),"\n";

print $svg->rect(-x => $conf->{CLUST_INDICATE}->{box_x},
                 -y => $conf->{CLUST_INDICATE}->{y_repeat}-$height_indicator,
                 -width => $conf->{CLUST_INDICATE}->{box_width},
                 -height => $height_indicator,
                 -style  => $conf->{REPEAT_CLUST}->{style}),"\n";
print $svg->text(-x => $conf->{CLUST_INDICATE}->{x}, 
                 -y => $conf->{CLUST_INDICATE}->{y_repeat}, 
                 -text =>sprintf("%s (window=%d kb)",$REPEAT_TYPE,$WINDOW_SIZE),
                 -font => $conf->{FONT},
                 -size => $conf->{CLUST_INDICATE}->{font_size}),"\n";

my $chr_idx = 0;
foreach my $tmp_chr (@{$conf_species->{CHR}}) {
    my $x_label = $conf->{CHR_LABEL}->{x};
    my $y_label = $conf->{CHR_LABEL}->{y_init} + $chr_idx*$conf->{CHR_STEP_Y};
    
    ## CHROMOSOME LABEL
    print "\n",$svg->comment("Chromosome Label : ".$tmp_chr),"\n";
    print $svg->text(   -id => 'label_chr'.$tmp_chr, 
                        -x => $x_label, -y => $y_label,
                        -font => $conf->{FONT}, 
                        -size => $conf->{CHR_LABEL}->{font_size},
                        -text => 'Chr '.$tmp_chr),"\n";

    ## CHROMOSOME BOX
    print "\n",$svg->comment("Chromosome Box : ".$tmp_chr),"\n";
    my $x_chr_box = $conf->{CHR_BOX}->{x};
    my $y_chr_box = $conf->{CHR_BOX}->{y_init} + $chr_idx*$conf->{CHR_STEP_Y};
    my $width_chr_box = $conf_species->{CHR_LENGTH}->{$tmp_chr}
                        / $conf->{CHR_BOX_X_SCALE};
    print $svg->rect( -id => 'chr_box_'.$tmp_chr,
                    -x => $x_chr_box, -y => $y_chr_box, 
                    -width => $width_chr_box,
                    -height => $conf->{CHR_BOX}->{height},
                    -style => $conf->{CHR_BOX}->{'style'}),"\n";

    ## Y GRID on CHROMOSOME BOX
    print "\n",$svg->comment("Y Grid on Chromosome Box : ".$tmp_chr),"\n";
    my $y_chr_box_grid = $y_chr_box + $conf->{CHR_BOX_Y}->{y_step};
    my $y_idx = 0;
    while($y_chr_box_grid < $y_chr_box + $conf->{CHR_BOX}->{height}) {
        print $svg->hline(-x => $x_chr_box, -y => $y_chr_box_grid, 
                        -length => $width_chr_box, 
                        -style => $conf->{CHR_BOX_Y}->{'line_style'}),"\n";
        my $x_grid_label = $x_chr_box + $conf->{CHR_BOX_Y}->{x_margin};
        my $y_grid_label = $y_chr_box_grid + $conf->{CHR_BOX_Y}->{y_margin}
                            + ($conf->{CHR_BOX_Y}->{y_step}/2);
        print $svg->text(-x => $x_grid_label, -y => $y_grid_label,
                        -font => $conf->{FONT},
                        -size=> $conf->{CHR_BOX_Y}->{'label_font_size'},
                        -text => $conf->{CHR_BOX_Y}->{label}->[$y_idx]),"\n";
        $y_chr_box_grid += $conf->{CHR_BOX_Y}->{y_step};
        $y_idx++;
    }

    ## X GRID ON CHROMOSOME BOX
    print "\n",$svg->comment("X Grid on Chromosome Box : ".$tmp_chr),"\n";
    my $x_grid_step = $conf->{CHR_BOX_X_STEP}/ $conf->{CHR_BOX_X_SCALE}; 
    my $x_chr_box_grid = $x_chr_box + $x_grid_step;
    while($x_chr_box_grid < $x_chr_box + $width_chr_box) {
        print $svg->vline(-x => $x_chr_box_grid, -y => $y_chr_box, 
                        -length => $conf->{CHR_BOX}->{height}, 
                        -style => $conf->{CHR_BOX_X}->{'line_style'}),"\n";
        my $x_grid_label = sprintf("%d M",($x_chr_box_grid - $x_chr_box) 
                            * $conf->{CHR_BOX_X_SCALE} / 1000000);
        print $svg->text(-x => $x_chr_box_grid+$conf->{CHR_BOX_X}->{x_margin}, 
                        -y => $y_chr_box + $conf->{CHR_BOX_X}->{y_margin},
                        -font => $conf->{FONT},
                        -size=> $conf->{CHR_BOX_X}->{'label_font_size'},
                        -text => $x_grid_label),"\n";
        $x_chr_box_grid += $x_grid_step;
    }

    ## VALUES
    print "\n",$svg->comment("VALUES OF DATA : ".$tmp_chr),"\n";
    foreach my $tmp_name (sort {$rv->{$tmp_chr}->{$a}->[0]->{start} 
                                <=> $rv->{$tmp_chr}->{$b}->[0]->{start}}
                                keys %{$rv->{$tmp_chr}}) {
        foreach my $tmp (sort {$b->{start} <=> $a->{start}}
                            @{$rv->{$tmp_chr}->{$tmp_name}}) {
                my $id = sprintf("%s:%d",$tmp_name, $tmp->{start});
                my $x_pt = $conf->{CHR_BOX}->{x} 
                            + $tmp->{start} / $conf->{CHR_BOX_X_SCALE};
                my $y_pt = $conf->{CHR_BOX}->{y_init}
                                + $chr_idx*$conf->{CHR_STEP_Y}
                                + $conf->{CHR_BOX}->{height} / 2;
                if( $y_pt < $conf->{CHR_BOX}->{y_init} 
                                + $chr_idx * $conf->{CHR_STEP_Y} ) {
                    $y_pt = $conf->{CHR_BOX}->{y_init} 
                                + $chr_idx * $conf->{CHR_STEP_Y};
                }

                print $svg->circle(-x => $x_pt, -y => $y_pt,
                                   -id => $id,
                                   -r => $conf->{DATA_POINT}->{r},
                                   -style => $conf->{DATA_POINT}->{style},
                                   -fill => $color_value ),"\n";
        }
    }

    ## Gene Cluster
    print "\n",$svg->comment("Gene Cluster : ".$tmp_chr),"\n";
    my $clust_width = $WINDOW_SIZE * 1000 / $conf->{CHR_BOX_X_SCALE};
    my $clust_bg_y = $conf->{GENE_CLUST}->{y_init}+$chr_idx*$conf->{CHR_STEP_Y};
    print $svg->text(-x => $conf->{CLUST_LABEL}->{x},
                     -y => $clust_bg_y + $conf->{CHR_BOX_Y}->{label_font_size},
                     -font => $conf->{FONT},
                     -size => $conf->{CHR_BOX_Y}->{label_font_size},
                     -fill => $conf->{GENE_CLUST}->{color},
                     -text => $max_gene_count ),"\n";
    print $svg->rect(-x => $x_chr_box, -y => $clust_bg_y,
                    -width => $width_chr_box,
                    -height => $conf->{GENE_CLUST}->{max_height},
                    -style => $conf->{GENE_CLUST}->{'background_style'}),"\n";

    foreach my $tmp_pos (sort { $a <=> $b } 
                            keys %{$gene_clust->{$tmp_chr}->{data}}) {
        my $clust_x = $conf->{GENE_CLUST}->{'x'}
                        + $tmp_pos / $conf->{CHR_BOX_X_SCALE};
        my $tmp_width = $clust_width;
        if( $clust_x + $tmp_width > $width_chr_box+$conf->{CHR_BOX}->{x}) {
            $tmp_width = $width_chr_box + $conf->{CHR_BOX}->{x} - $clust_x ;
        }
        my $clust_height = $gene_clust->{$tmp_chr}->{data}->{$tmp_pos}
                        / $max_gene_count
                        * $conf->{GENE_CLUST}->{max_height};
        my $clust_y = $conf->{GENE_CLUST}->{max_height}
                        + $conf->{GENE_CLUST}->{y_init} 
                        + $conf->{CHR_STEP_Y}*$chr_idx - $clust_height;
        print $svg->rect(-x => $clust_x, -y => $clust_y, 
                        -width => $tmp_width , -height => $clust_height, 
                        -style => $conf->{GENE_CLUST}->{'style'}),"\n";
    }

    ## Repeat Cluster
    print "\n",$svg->comment("Repeat Cluster : ".$tmp_chr),"\n";
    
    ### Background
    $clust_bg_y = $conf->{REPEAT_CLUST}->{y_init}+$chr_idx*$conf->{CHR_STEP_Y};
    print $svg->text(-x => $conf->{CLUST_LABEL}->{x},
                     -y => $clust_bg_y + $conf->{CHR_BOX_Y}->{label_font_size},
                     -font => $conf->{FONT},
                     -size => $conf->{CHR_BOX_Y}->{label_font_size},
                     -fill => $conf->{REPEAT_CLUST}->{color},
                     -text => $max_repeat_count ),"\n";
    print $svg->rect(-x => $x_chr_box, -y => $clust_bg_y,
                    -width => $width_chr_box,
                    -height => $conf->{REPEAT_CLUST}->{max_height},
                    -style => $conf->{REPEAT_CLUST}->{'background_style'}),"\n";

    foreach my $tmp_pos (sort { $a <=> $b } 
                            keys %{$rep_clust->{$tmp_chr}->{data}}) {
        my $clust_x = $conf->{REPEAT_CLUST}->{'x'}
                        + $tmp_pos / $conf->{CHR_BOX_X_SCALE};
        my $tmp_width = $clust_width;
        if( $clust_x + $tmp_width > $width_chr_box+$conf->{CHR_BOX}->{x}) {
            $tmp_width = $width_chr_box + $conf->{CHR_BOX}->{x} - $clust_x ;
        }
        my $clust_height = 0;
        if( $rep_clust->{$tmp_chr}->{'max'} > 0 ) {
            $clust_height = $rep_clust->{$tmp_chr}->{data}->{$tmp_pos}
                        / $max_repeat_count
                        * $conf->{REPEAT_CLUST}->{max_height};
        }
        my $clust_y = $conf->{REPEAT_CLUST}->{max_height} 
                        + $conf->{REPEAT_CLUST}->{y_init} 
                        + $conf->{CHR_STEP_Y}*$chr_idx - $clust_height;
        print $svg->rect(-x => $clust_x, -y => $clust_y, 
                        -width => $tmp_width, -height => $clust_height, 
                        -style => $conf->{REPEAT_CLUST}->{'style'}),"\n";
    }

    print "\n";
    $chr_idx++;
}

## Print FOOTER
print $svg->footer(),"\n";
