#!/usr/bin/perl -w
use strict;

## TODO 
## max & min function (under construction)

use lib "../libperl";
use GenomeMap::SVG;
use GenomeMap::IO;
use GenomeMap::Map;
use CGI;

my $svg = new GenomeMap::SVG;
my $io  = new GenomeMap::IO;
my $map = new GenomeMap::Map;
my $q   = new CGI;

my $SPECIES = "Celegans";
my $COLOR = 'maroon';
my $WITH_ERROR = 'TRUE';

my $conf = do "../etc/hist.pl";
my $conf_sp = do "../etc/$SPECIES.pl";

#my $file_data   = '../tmp/topTable_C1.txt';
#my $file_data   = '../tmp/eset_A1.txt';
my $file_data   = '../tmp/'.$q->param('file');
my $file_map    = "../data/$SPECIES.oligo_set.WS150";

my $p_value     = 1.00;
if( $q->param('P') ) { $p_value = $q->param('P'); }

my $file_type = 'eset';
if( $q->param('type') ) { $file_type = $q->param('type'); }

unless(-e $file_data) {
    die "NO DATA FILE : ",$file_data,"\n";
}

my $data_value  = undef;
if( $file_type eq 'topTable' ) {
    $data_value  = $io->read_topTable(-file => $file_data, 
                                        -value => 'M', -P => $p_value );
} elsif ( $file_type eq 'eset' ) {
    $data_value  = $io->read_eset(-file => $file_data), 
} else {
    $data_value  = $io->read_eset(-file => $file_data), 
}

my $data_map    = $io->read_map(-file => $file_map);

my ($r_sp, $rv) = $map->summary_by_chr( -data => $data_value, 
                                        -map => $data_map);
my $max_value = $r_sp->{max};
my $min_value = $r_sp->{min};
if( $q->param('max') ) { $max_value = $q->param('max'); }
if( $q->param('min') ) { $min_value = $q->param('min'); }
my $max_height = $max_value - $min_value;

#print STDERR $max_height,":",$max_value,":",$min_value,"\n";
#foreach my $tmp_chr (keys %{$rv}) {
#    print STDERR $tmp_chr,":",$rv->{$tmp_chr}->{mean},"\n";
#}

my $chr_number = $#{$conf_sp->{CHR}} + 1;
my $width = $conf->{FRAME}->{x_init} + $conf->{X_STEP}*$chr_number 
            + $conf->{X_MARGIN};

## HEADER
print $svg->http_header();
print $svg->header( -width => $width, -height => $conf->{HEIGHT}),"\n";

## BORDER FRAME
print $svg->rect( -x => $conf->{FRAME}->{x_init}, -y => $conf->{FRAME}->{y},
                  -width => $conf->{X_STEP} * $chr_number,
                  -height => $conf->{FRAME}->{height}, 
                  -style => $conf->{FRAME}->{style} ),"\n";

my $idx = 0;
foreach my $tmp_chr (@{$conf_sp->{CHR}}) {
    ## LABEL
    print $svg->text( -x => $conf->{LABEL}->{x_init} + $idx * $conf->{X_STEP},
                      -y => $conf->{LABEL}->{y}, 
                      -font => $conf->{FONT}, 
                      -size => $conf->{LABEL}->{font_size},
                      -text => $tmp_chr ),"\n";
    
    ## FRAME
    my $value_scaled = $conf->{BAR}->{y_max} 
                        * ($rv->{$tmp_chr}->{mean} - $min_value)
                        / ($max_height - $min_value);
    my $error_scaled = $conf->{BAR}->{y_max} 
                        * ($rv->{$tmp_chr}->{error} - $min_value) 
                        / ($max_height - $min_value );

=rem
    print STDERR $tmp_chr,":",$r_sp->{max},"\t",$r_sp->{min},"\t",$max_height,"\n";
    print STDERR $rv->{$tmp_chr}->{mean},"\t",$rv->{$tmp_chr}->{error},"\n";
    print STDERR $value_scaled,"\t",$error_scaled,"\n";
    next;
=cut

    my $x = $conf->{BAR}->{x_init} + $idx * $conf->{X_STEP};
    my $y = $conf->{FRAME}->{y} + $conf->{BAR}->{y_max} - $value_scaled;
    my $y_actual = $y;
    if( $y < $conf->{FRAME}->{y} ) { 
        $value_scaled -= ($conf->{FRAME}->{y} - $y); 
        $y_actual = $conf->{FRAME}->{y}; 
    }
    
    print $svg->rect(-x => $x, -y => $y_actual,
                     -width => $conf->{BAR}->{width}, 
                     -height => $value_scaled,
                     -style => $conf->{BAR}->{style}, -fill => $COLOR ),"\n";
    
    print $svg->text(-x => $conf->{BAR}->{x_value_init} + $idx*$conf->{X_STEP},
                     -y => $y_actual - $conf->{BAR}->{value_y_margin},
                     -font => $conf->{FONT},
                     -size => $conf->{BAR}->{value_font_size}, 
                     -text => sprintf("%.2f",$rv->{$tmp_chr}->{mean}) ),"\n";
                     
    if( $WITH_ERROR eq 'TRUE' ) {
        my $y_error = $y - $error_scaled;
        if( $y_error < $conf->{FRAME}->{y} ) {
            $error_scaled -= ($conf->{FRAME}->{y} - $y_error);
            $y_error = $conf->{FRAME}->{y};
            print $svg->vline(-x =>$conf->{ERROR}->{x_init}+$idx*$conf->{X_STEP},
                          -y => $y_error, -length => $error_scaled, 
                          -style => $conf->{ERROR}->{style}),"\n";
        } else {
            print $svg->vline(-x =>$conf->{ERROR}->{x_init}+$idx*$conf->{X_STEP},
                          -y => $y_error, -length => $error_scaled, 
                          -style => $conf->{ERROR}->{style}),"\n";
            print $svg->hline(-x => $conf->{ERROR}->{x_init} 
                                - $conf->{ERROR}->{end_tick_width}/2
                                + $idx * $conf->{X_STEP},
                          -y => $y-$error_scaled, 
                          -length => $conf->{ERROR}->{end_tick_width},
                          -style => $conf->{ERROR}->{style}),"\n";
        }
    }
    
    $idx++;
}
print $svg->footer(),"\n";
