package GenomeMap::IO;

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
}

sub read_list {
    my ($self, %param) = @_;
    my $rv = undef;

    _check_file(%param);

    open(LIST,$param{-file});
    <LIST>;
    while(<LIST>) {
        chomp;
        s/\"//g;
        my ($id, @tmp) = split(/\s+/);
        $rv->{$id}->{value} = 1;
    }
    close(LIST);

    return $rv;
}
            
sub read_eset {
    my ($self, %param) = @_;
    my $rv = undef;

    _check_file(%param);

    open(TOP,$param{-file});
    <TOP>;
    while(<TOP>) {
        chomp;
        s/\"//g;
        my ($id, $value) = split(/\s+/);
        $rv->{$id}->{value} = $value;
    }
    close(TOP);

    return $rv;
}
            
sub read_topTable {
    my ($self, %param) = @_;
    my $rv = undef;

    _check_file(%param);

    open(TOP,$param{-file});
    <TOP>;
    while(<TOP>) {
        chomp;
        s/\"//g;
        my ($id, $probe_set, $M, $A, $t, $P, $B) = split(/\s+/);
        #if( $param{-P} && $P > $param{-P} ) { next; }

        if( $param{-value} eq 't' ) {
            $rv->{$probe_set} = { value => $t, P => $P };
        } elsif( $param{-value} eq 'A' ) {
            $rv->{$probe_set} = { value => $A, P => $P };
        } else {
            ## Default : M value
            $rv->{$probe_set} = { value => $M, P => $P };
        }
    }
    close(TOP);

    return $rv;
}

sub read_map {
    my ($self, %param) = @_;
    my $rv = undef;

    _check_file(%param);
    
    open(MAP, $param{-file});
    while(<MAP>) {
        chomp;
        my ($name, $chr, $pos) = split(/\s+/);
        $rv->{$name}->{$chr}->{$pos} = 1;
    }
    close(MAP);

    return $rv;
}

sub read_chr_stat {
    my ($self,$filename) = @_;
    my $rv;

    open(LIST,$filename);
    while(<LIST>) {
        chomp;
        my ($chr, $pos, $count) = split(/\t/);
        $rv->{$chr}->{data}->{$pos} = $count;
        if( not exists $rv->{$chr}->{min} ) { $rv->{$chr}->{min} = $count; }
        if( not exists $rv->{$chr}->{max} ) { $rv->{$chr}->{max} = $count; }
        if( $rv->{$chr}->{max} < $count ) { $rv->{$chr}->{max} = $count; }
        if( $rv->{$chr}->{min} > $count ) { $rv->{$chr}->{min} = $count; }
    }
    close(LIST);
    return $rv;
}

sub _check_file {
    my (%param) = @_;
    if( not exists $param{-file} )  { return undef; }
    unless( -e $param{-file} )      { return undef; }
}

1;
