package GenomeMap::Map;

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
}

sub order_by_chr {
    my ($self, %param) = @_;
    my $rv = undef;

    foreach my $tmp_name (keys %{$param{-data}}) {
        if( exists $param{-map}->{$tmp_name} ) {
            foreach my $tmp_chr (keys %{$param{-map}->{$tmp_name}}) {
                foreach my $tmp_pos 
                        (keys %{$param{-map}->{$tmp_name}->{$tmp_chr}}) {
                    #print STDERR $tmp_chr,":",$tmp_name,":",$tmp_pos,":",
                    #        $param{-data}->{$tmp_name}->{value},"\n";
                    push(@{$rv->{$tmp_chr}->{$tmp_name}}, 
                            { start => $tmp_pos,
                              value => $param{-data}->{$tmp_name}->{value},
                              Pvalue => $param{-data}->{$tmp_name}->{P} });
                }
            }
        }
    }

    return $rv;
}

sub summary_by_chr {
    my ($self, %param) = @_;
    my $rv = undef;
    my $rv_special = undef;

    foreach my $tmp_name (keys %{$param{-data}}) {
        if( exists $param{-map}->{$tmp_name} ) {
            foreach my $tmp_chr (keys %{$param{-map}->{$tmp_name}}) {
                push(@{$rv->{$tmp_chr}->{raw}}, 
                        $param{-data}->{$tmp_name}->{value});
            }
        } else {
            push(@{$rv_special->{orphant}}, $tmp_name);
        }
    }

    my ($max,$min) = (0,0);
    foreach my $tmp_chr (keys %{$rv}) {
        if(exists $rv->{$tmp_chr}->{raw}) {
            $rv->{$tmp_chr}->{mean} = &_calc_mean( $rv->{$tmp_chr}->{raw} );
            $rv->{$tmp_chr}->{error} = &_calc_std_dev( $rv->{$tmp_chr}->{raw} );

            if( not defined $max || not defined $min ) { 
                $max = $rv->{$tmp_chr}->{mean} + $rv->{$tmp_chr}->{error};
                $min = $rv->{$tmp_chr}->{mean} - $rv->{$tmp_chr}->{error};
            }
            
            if( $max < $rv->{$tmp_chr}->{mean} + $rv->{$tmp_chr}->{error}) {
                $max = $rv->{$tmp_chr}->{mean} + $rv->{$tmp_chr}->{error};
            }

            if( $min > $rv->{$tmp_chr}->{mean} - $rv->{$tmp_chr}->{error}) {
                $min = $rv->{$tmp_chr}->{mean} - $rv->{$tmp_chr}->{error};
            }
        }
    }
    $rv_special->{min} = $min; $rv_special->{max} = $max;

    return ($rv_special,$rv);
}

sub _calc_mean {
    my $array_ref = shift;
    my $rv = 0;
    
    foreach my $tmp (@{$array_ref}) { $rv += $tmp; }
    return $rv / ($#{$array_ref} + 1);
}

sub _calc_std_dev {
    my $array_ref = shift;
    
    my $mean = 0;
    foreach my $tmp (@{$array_ref}) { $mean += $tmp; }
    $mean = $mean / ($#{$array_ref} + 1);

    my $var = 0;
    foreach my $tmp (@{$array_ref}) { $var += ($tmp - $mean)**2; }
    $var = $var / $#{$array_ref};

    return sqrt($var);
}

sub _calc_std_err {
    my $array_ref = shift;
    
    my $mean = 0;
    foreach my $tmp (@{$array_ref}) { $mean += $tmp; }
    $mean = $mean / ($#{$array_ref} + 1);

    my $var = 0;
    foreach my $tmp (@{$array_ref}) { $var += ($tmp - $mean)**2; }
    $var = $var / $#{$array_ref};

    return sqrt($var / ($#{$array_ref}+1) );
}

1;
