package GenomeMap::SVG;

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;
}

sub http_header {
    my $self = shift;
    return "Content-type: image/svg+xml\r\n\r\n";
}

sub header {
    my ($self,%param) = @_;
    if(not exists $param{-width} || not exists $param{-height}) {
        die "No WIDTH and/or HEIGHT info for GenomeMap::SVG::header()\n";
    }

    return <<__SVG_HEADER__;
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 20001102//EN"
"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd">
<svg width="$param{-width}" height="$param{-height}" version="1.1" xmlns="http://www.w3.org/2000/svg">
__SVG_HEADER__
}

sub footer {
    my $self = shift;
    return '</svg>';
}

sub comment {
    my ($self, $comment) = @_;
    return '<!-- '.$comment.' -->';
}

sub text {
    my ($self, %param) = @_;
    if(not exists $param{-x} || not exists $param{-y}
        || not exists $param{-text} || not exists $param{-size}
        || not exists $param{-font} ) {
        die "No X,Y, or TEXT info for GenomeMap::SVG::text()\n";
    }

    my $rv = '<text x="'.$param{-x}.'" y="'.$param{-y}.'" ';
    if(exists $param{-fill}) {
        $rv .= ' fill="'.$param{-fill}.'" ';
    }
    $rv .= 'font-family="'.$param{-font}.'" font-size="'.$param{-size}.'">';
    $rv .= $param{-text}. '</text>';

    return $rv;
}

sub rect {
    my ($self, %param) = @_;
    if(not exists $param{-x} || not exists $param{-y}
        || not exists $param{-width} || not exists $param{-height} ) {
        die "No X,Y,WIDTH or HEIGHT info for GenomeMap::SVG::rect()\n";
    }

    $param{-x} = _re_digit($param{-x});
    $param{-y} = _re_digit($param{-y});
    $param{-height} = _re_digit($param{-height});
    $param{-width} = _re_digit($param{-width});

    my $rv = '<rect x="'.$param{-x}.'" y="'.$param{-y}.'" width="'
            .$param{-width}.'" height="'.$param{-height}.'" ';
    $rv .= &_add_id_and_style(%param);
    $rv .= '/>';

    return $rv;
}

sub circle {
    my ($self, %param) = @_;
    if(not exists $param{-x} || not exists $param{-y} 
        || not exists $param{-r} ) {
        die "No X,Y, or R info for GenomeMap::SVG::circle()\n";
    }

    $param{-x} = _re_digit($param{-x});
    $param{-y} = _re_digit($param{-y});

    my $rv = '<circle cx="'.$param{-x}.'" cy="'.$param{-y}
            .'" r="'.$param{-r}.'" ';
    $rv .= &_add_id_and_style(%param);
    $rv .= '/>';

    return $rv;
}

sub hline {
    my ($self, %param) = @_;
    if(not exists $param{-x} || not exists $param{-y}
        || not exists $param{-length} ) {
        die "No X,Y or WIDTH info for GenomeMap::SVG::hline()\n";
    }

    my $x2 = $param{-x} + $param{-length};
    my $rv = '<line x1="'.$param{-x}.'" y1="'.$param{-y}.
            '" x2="'.$x2.'" y2="'.$param{-y}.'" ';
    $rv .= &_add_id_and_style(%param);
    $rv .= '/>';
}

sub vline {
    my ($self, %param) = @_;
    if(not exists $param{-x} || not exists $param{-y}
        || not exists $param{-length} ) {
        die "No X,Y or LENGTH info for GenomeMap::SVG::vline()\n";
    }

    my $y2 = $param{-y} + $param{-length};
    my $rv = '<line x1="'.$param{-x}.'" y1="'.$param{-y}.
            '" x2="'.$param{-x}.'" y2="'.$y2.'" ';
    $rv .= &_add_id_and_style(%param);
    $rv .= '/>';
}

sub _add_id_and_style {
    my (%param) = @_;
    my $rv = '';

    if(exists $param{-id})      { $rv .= 'id="'.$param{-id}.'" '; }
    if(exists $param{-style})   { $rv .= 'style="'.$param{-style}.'" '; }
    if(exists $param{-fill})    { $rv .= 'fill="'.$param{-fill}.'" '; }

    return $rv;
}

sub _re_digit {
    my $arg = shift;
    return sprintf("%.2f",$arg);
}


1;
