#!/usr/bin/perl -w
use strict;

use lib "../libperl";
use GenomeMap::SVG;
use GenomeMap::IO;

my $svg = new GenomeMap::SVG;
my $io  = new GenomeMap::IO;

my $SPECIES = "celegans";

my $conf = do "../etc/hist.pl";
my $conf_sp = do "../etc/celegans.pl";

my $COLOR = 'red';
my %VALUE = ( 'I' => 200, 'II' => 25, 'III' => 142,
              'IV' => 204, 'V' => 140, 'X' => 100 );
my %ERR = ( 'I' => 10, 'II' => 5, 'III' => 20, 
              'IV' => 24, 'V' => 40, 'X' => 10 );
my $WITH_ERROR = 'TRUE';

my $max_value = 0;
foreach my $tmp_chr (@{$conf_sp->{CHR}}) {
    if($max_value < $VALUE{$tmp_chr}+$ERR{$tmp_chr}) {
        $max_value = $VALUE{$tmp_chr}+$ERR{$tmp_chr};
    }
}

my $chr_number = $#{$conf_sp->{CHR}} + 1;
my $width = $conf->{BOX}->{x_init} + $conf->{X_STEP}*$chr_number;

## HEADER
print $svg->http_header();
print $svg->header( -width => $width, -height => $conf->{HEIGHT}),"\n";

## BORDER BOX
print $svg->rect( -x => $conf->{BOX}->{x_init}, -y => $conf->{BOX}->{y},
                  -width => $conf->{X_STEP} * $chr_number,
                  -height => $conf->{BOX}->{height}, 
                  -style => $conf->{BOX}->{style} ),"\n";

my $idx = 0;
foreach my $tmp_chr (@{$conf_sp->{CHR}}) {
    ## LABEL
    print $svg->text( -x => $conf->{LABEL}->{x_init} + $idx * $conf->{X_STEP},
                      -y => $conf->{LABEL}->{y}, 
                      -font => $conf->{FONT}, 
                      -size => $conf->{LABEL}->{font_size},
                      -text => $tmp_chr ),"\n";
    
    ## BOX
    my $value_scaled = $conf->{BAR}->{y_max} * $VALUE{$tmp_chr} /$max_value;
    my $error_scaled = $conf->{BAR}->{y_max} * $ERR{$tmp_chr} /$max_value;
    my $x = $conf->{BAR}->{x_init} + $idx * $conf->{X_STEP};
    my $y = $conf->{BOX}->{y} + $conf->{BAR}->{y_max} - $value_scaled;
    
    print $svg->rect(-x => $x, -y => $y,
                     -width => $conf->{BAR}->{width}, 
                     -height => $value_scaled,
                     -style => $conf->{BAR}->{style}, -fill => $COLOR ),"\n";
    
    print $svg->text(-x => $conf->{BAR}->{x_value_init} + $idx*$conf->{X_STEP},
                     -y => $y,
                     -font => $conf->{FONT},
                     -size => $conf->{BAR}->{value_font_size}, 
                     -text => $VALUE{$tmp_chr}),"\n";
                     
    if( $WITH_ERROR eq 'TRUE' ) {
        print $svg->vline(-x => $conf->{ERROR}->{x_init}+$idx*$conf->{X_STEP},
                          -y => $y-$error_scaled, -length => $error_scaled, 
                          -style => $conf->{ERROR}->{style}),"\n";
        print $svg->hline(-x => $conf->{ERROR}->{x_init} 
                                - $conf->{ERROR}->{end_tick_width}/2
                                + $idx * $conf->{X_STEP},
                          -y => $y-$error_scaled, 
                          -length => $conf->{ERROR}->{end_tick_width},
                          -style => $conf->{ERROR}->{style}),"\n";

    }
    
    $idx++;
}
print $svg->footer(),"\n";
