#!/usr/bin/perl -w
use strict;

use lib "../lib";
use GenomeMap::SVG;
use GenomeMap::IO;

my $svg = new GenomeMap::SVG;
my $io  = new GenomeMap::IO;

my $conf = do "../etc/venn3.pl";

my $LABEL1 = "lin-15A";
my $LABEL2 = "lin-15A + lin-15B";
my $LABEL3 = "lin-15B";

my $VALUE1 = 15;
my $VALUE2 = 20;
my $VALUE3 = 30;
my $VALUE12 = 254;
my $VALUE13 = 128;
my $VALUE23 = 64;
my $VALUE123 = 50;

my $COLOR1 = 'red';
my $COLOR2 = 'blue';
my $COLOR3 = 'green';
my $COLOR12 = 'maroon';
my $COLOR13 = 'orange';
my $COLOR23 = 'purple';
my $COLOR123 = 'aqua';

## HEADER
print $svg->http_header();
print $svg->header( -width => $conf->{WIDTH}, -height => $conf->{HEIGHT}),"\n";

## TITLE
#print $svg->text( -x => $conf->{TITLE}->{'x'}, -y => $conf->{TITLE}->{'y'},
#                  -text => $TITLE, -size => $conf->{TITLE}->{'font_size'},
#                  -font => $conf->{FONT}),"\n";

## LABEL
print $svg->text( -x => $conf->{ITEM1_LABEL}->{'x'}, 
                  -y => $conf->{ITEM1_LABEL}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{LABEL_FONT_SIZE},
                  -text => $LABEL1 ),"\n";
print $svg->text( -x => $conf->{ITEM2_LABEL}->{'x'}, 
                  -y => $conf->{ITEM2_LABEL}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{LABEL_FONT_SIZE},
                  -text => $LABEL2 ),"\n";
print $svg->text( -x => $conf->{ITEM3_LABEL}->{'x'}, 
                  -y => $conf->{ITEM3_LABEL}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{LABEL_FONT_SIZE},
                  -text => $LABEL3 ),"\n";

## BOX
print $svg->rect(-x => $conf->{ITEM1_BOX}->{x}, -y => $conf->{ITEM1_BOX}->{y},
                 -width => $conf->{ITEM1_BOX}->{width},
                 -height => $conf->{ITEM1_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR1),"\n";
print $svg->rect(-x => $conf->{ITEM2_BOX}->{x}, -y => $conf->{ITEM2_BOX}->{y},
                 -width => $conf->{ITEM2_BOX}->{width},
                 -height => $conf->{ITEM2_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR2),"\n";
print $svg->rect(-x => $conf->{ITEM3_BOX}->{x}, -y => $conf->{ITEM3_BOX}->{y},
                 -width => $conf->{ITEM3_BOX}->{width},
                 -height => $conf->{ITEM3_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR3),"\n";

print $svg->rect(-x => $conf->{ITEM12_BOX}->{x}, -y => $conf->{ITEM12_BOX}->{y},
                 -width => $conf->{ITEM12_BOX}->{width},
                 -height => $conf->{ITEM12_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR12),"\n";
print $svg->rect(-x => $conf->{ITEM13_BOX}->{x}, -y => $conf->{ITEM13_BOX}->{y},
                 -width => $conf->{ITEM13_BOX}->{width},
                 -height => $conf->{ITEM13_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR13),"\n";
print $svg->rect(-x => $conf->{ITEM23_BOX}->{x}, -y => $conf->{ITEM23_BOX}->{y},
                 -width => $conf->{ITEM23_BOX}->{width},
                 -height => $conf->{ITEM23_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR23),"\n";

print $svg->rect(-x => $conf->{ITEM123_BOX}->{x}, 
                 -y => $conf->{ITEM123_BOX}->{y},
                 -width => $conf->{ITEM123_BOX}->{width},
                 -height => $conf->{ITEM123_BOX}->{height},
                 -style => $conf->{BOX_STYLE}, -fill => $COLOR123),"\n";
## VALUE
print $svg->text( -x => $conf->{ITEM1_VALUE}->{'x'}, 
                  -y => $conf->{ITEM1_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE1 ),"\n";
print $svg->text( -x => $conf->{ITEM2_VALUE}->{'x'}, 
                  -y => $conf->{ITEM2_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE2 ),"\n";
print $svg->text( -x => $conf->{ITEM3_VALUE}->{'x'}, 
                  -y => $conf->{ITEM3_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE3 ),"\n";
print $svg->text( -x => $conf->{ITEM12_VALUE}->{'x'}, 
                  -y => $conf->{ITEM12_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE12 ),"\n";
print $svg->text( -x => $conf->{ITEM13_VALUE}->{'x'}, 
                  -y => $conf->{ITEM13_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE13 ),"\n";
print $svg->text( -x => $conf->{ITEM23_VALUE}->{'x'}, 
                  -y => $conf->{ITEM23_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE23 ),"\n";
print $svg->text( -x => $conf->{ITEM123_VALUE}->{'x'}, 
                  -y => $conf->{ITEM123_VALUE}->{'y'},
                  -font => $conf->{FONT}, -size => $conf->{VALUE_FONT_SIZE},
                  -text => $VALUE123 ),"\n";

## FOOTER
print $svg->footer(),"\n";
