#!/usr/bin/perl -w
use strict;

## Gene
## locusName(CGCName), WBGeneID, seqName, conciseDescription, chromosome,
## strand, genomeGeneStart, genomeGeneEnd, sourceDB

## Protein
## WBGeneID, WBPepID, transcriptName, breifIdentification, ORFLength,
## sourceDB

my $conf = do "../etc/wormi.pl";
my @tmp_ts = localtime(time());
my $timestamp = sprintf("%02d%02d%02d",$tmp_ts[3],$tmp_ts[4]+1,$tmp_ts[5]%100);

my $WS_GENE_FILE = $conf->{DIR_WORMI}. $conf->{DIR_WORMBASE_DATA} 
                . sprintf($conf->{FILE_WS_GENEID}, $conf->{WORMBASE_VER});
my $WS_FUNC_FILE = $conf->{DIR_WORMI}. $conf->{DIR_WORMBASE_DATA} 
                . sprintf($conf->{FILE_WS_FUNCTION}, $conf->{WORMBASE_VER});
my $WS_GFF_FILE = $conf->{DIR_WORMI}. $conf->{DIR_WORMBASE_DATA} 
                . sprintf($conf->{FILE_WS_GFF}, $conf->{WORMBASE_VER});
my $WS_CDS_FILE = $conf->{DIR_WORMI}. $conf->{DIR_WORMBASE_DATA} 
                . $conf->{FILE_WS_CDS} .".". $conf->{WORMBASE_VER}. ".txt";

my $GENE_OUT_FILE = $conf->{DIR_WORMI}. $conf->{DIR_WORMI_DATA}
                . $conf->{FILE_WORMI_GENE}.".".$timestamp."."
                . $conf->{WORMBASE_VER}.".txt";

my $PROT_OUT_FILE = $conf->{DIR_WORMI}. $conf->{DIR_WORMI_DATA}
                . $conf->{FILE_WORMI_PROTEIN}.".".$timestamp."."
                . $conf->{WORMBASE_VER}.".txt";

my %genes;
open(GENE,$WS_GENE_FILE);
print STDERR "Read ",$WS_GENE_FILE,"...\n";
while(<GENE>) {
    chomp;
    my ($wb, $cgc, $seqName) = split(/,/);
    $cgc = "-" unless($cgc);
    $seqName = "-" unless($seqName);
    
    $genes{$wb}->{CGCName} = $cgc;
    $genes{$wb}->{SEQName} = $seqName;
    $genes{$wb}->{desc} = "non_available";
}
close(GENE);

open(FUNC,$WS_FUNC_FILE);
print STDERR "Read ",$WS_FUNC_FILE,"...\n";
while(<FUNC>) {
    chomp;
    my @tmp = split(/\t/);
    $genes{$tmp[0]}->{desc} = $tmp[3];
    my @tmp_desc = split(/;/, $tmp[3]);
    $genes{$tmp[0]}->{breif_id} = $tmp_desc[0];
}

my $GFF_gene = $WS_GFF_FILE; $GFF_gene =~ s/\.gff/\.gene\.gff/;
my $GFF_CDS = $WS_GFF_FILE; $GFF_CDS =~ s/\.gff/\.CDS\.gff/;

open(GFF1,$GFF_gene);
print STDERR "Read ",$GFF_gene,"...\n";
while(<GFF1>) {
    chomp;
    my ($seqname, $source, $feature, $start, $end, $score, 
            $strand, $frame, $comment) = split(/\t/);

    next unless ($comment);

    if($comment =~ /Gene "(WBGene[0-9]+)"/) { 
        my $wb = $1;
        $genes{$wb}->{start} = $start;
        $genes{$wb}->{end} = $end;
        $genes{$wb}->{strand} = $strand;
        $genes{$wb}->{chr} = $seqname;
    }
}
close(GFF1);

open(GFF2, $GFF_CDS);
print STDERR "Read ",$GFF_CDS,"...\n";
while(<GFF2>) {
    chomp;
    my ($seqname, $source, $feature, $start, $end, $score, 
            $strand, $frame, $comment) = split(/\t/);

    next unless ($comment);

    if($comment =~ /Gene "(WBGene[0-9]+)"/) { 
        my $wb = $1;
        if( $comment =~ /WormPep "(WP:CE[0-9]+)"/ ) {
            push(@{$genes{$wb}->{wormpep}}, 
                        {id => $1, length => $end - $start + 1});
        }
    }
}
close(GFF2);

my %cds;
open(CDS, $WS_CDS_FILE);
print STDERR "Read ",$WS_CDS_FILE,"...\n";
while(<CDS>) {
    chomp;
    my ($WBPepID, $transcript, $briefID) = split(/\t/);
    next unless ($briefID);
    $cds{$WBPepID} = { transcript => $transcript, brief_id =>  $briefID };
}
close(CDS);

open(GENEO, ">$GENE_OUT_FILE");
print STDERR "Write ",$GENE_OUT_FILE,"...\n";
open(PROTO, ">$PROT_OUT_FILE");
print STDERR "Write ",$PROT_OUT_FILE,"...\n";
foreach my $tmp_wb (sort keys %genes) {
    my $g = $genes{$tmp_wb};
    next unless ($g->{chr});
    
    print GENEO $g->{CGCName},"\t",$tmp_wb,"\t",$g->{SEQName},"\t"
        ,$g->{desc},"\t", $g->{chr},"\t", $g->{strand},"\t"
        ,$g->{start},"\t", $g->{end},"\t"
        ,$conf->{WORMBASE_VER},"\n";

    foreach my $tmp_wp (@{$g->{wormpep}}) {
        my $tmp_pep_id = $tmp_wp->{id};
        print PROTO $tmp_wb,"\t", $tmp_wp->{id},"\t";
        if(exists $cds{$tmp_pep_id}) {
            print PROTO $cds{$tmp_pep_id}->{transcript},"\t"
                    , $cds{$tmp_pep_id}->{brief_id},"\t";
        } else {
            print PROTO "non_available\t", "non_available\t";
        }
        print PROTO $tmp_wp->{length},"\t"
            ,$conf->{WORMBASE_VER},"\n";
    }
}
close(GENEO);
close(PROTO);
